package cn.com.qmth.examcloud.core.basic.api.controller;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import cn.com.qmth.examcloud.commons.base.util.ErrorMsg;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelError;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.security.entity.AccessUser;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.Org;
import cn.com.qmth.examcloud.core.basic.service.bean.OrgDto;
import cn.com.qmth.examcloud.core.basic.service.impl.ExportService;
import cn.com.qmth.examcloud.core.basic.service.impl.OrgService;
import io.swagger.annotations.ApiOperation;

/**
 * 机构服务API
 * Created by songyue on 17/1/14.
 */
@RestController
@RequestMapping("${app.api.root}/org")
public class OrgController extends ControllerSupport{

    private static final Logger LOG = LoggerFactory.getLogger(OrgController.class);

    @Autowired
    OrgRepo orgRepo;

    @Autowired
    OrgService orgService;
    
	@Value("${staticResource.rootPath}")
	private String rootPath;
	
	@Autowired
	private JdbcTemplate jdbcTemplate;

    @ApiOperation(value = "查询机构分页带查询", notes = "分页带查询")
    @GetMapping("/all/{curPage}/{pageSize}")
    public ResponseEntity getAllOrg(@ModelAttribute Org orgCriteria,
                                    @PathVariable Integer curPage,
                                    @PathVariable Integer pageSize) {
        return new ResponseEntity(orgService.findAll(orgCriteria, new PageRequest(curPage - 1, pageSize)), HttpStatus.OK);
    }

    @ApiOperation(value = "查询顶层机构分页带查询", notes = "分页带查询")
    @GetMapping("/parent/{curPage}/{pageSize}")
    public ResponseEntity getParentOrg(@ModelAttribute Org orgCriteria,
                                       @PathVariable Integer curPage,
                                       @PathVariable Integer pageSize) {
        orgCriteria.setParentId(0L);
        orgCriteria.setRootId(0L);
        return new ResponseEntity(orgService.findAll(orgCriteria, new PageRequest(curPage - 1, pageSize)), HttpStatus.OK);
    }

    @ApiOperation(value = "查询机构不分页带查询", notes = "不分页带查询")
    @GetMapping("/alls/{type}")
    public ResponseEntity getAllExam(@PathVariable String type,
                                     HttpServletRequest request) {
        cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
        if (accessUser != null) {
        	if(type.equals("school")){
        		return new ResponseEntity(orgRepo.findAllSchoolByParentId(accessUser.getRootOrgId()), HttpStatus.OK);
        	}else {
        		return new ResponseEntity(orgRepo.findAllPrintByParentId(accessUser.getRootOrgId()), HttpStatus.OK);
			}
        }
        return new ResponseEntity(null, HttpStatus.OK);
    }

    @ApiOperation(value = "按ID查询机构", notes = "ID查询")
    @GetMapping("/{id}")
    public ResponseEntity<Org> getOrgById(@PathVariable Long id) {
        return new ResponseEntity(orgService.findOne(id), HttpStatus.OK);
    }

    @ApiOperation(value = "查询下属机构不带分页", notes = "不分页")
    @GetMapping("/sub/{parentId}")
    public ResponseEntity getOrgByParentId(@PathVariable Long parentId,HttpServletRequest request) {
        cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
        if(accessUser != null){
            if(accessUser.getRootOrgId().longValue() != accessUser.getOrgId().longValue()){
            	return new ResponseEntity(orgRepo.findById(accessUser.getOrgId()), HttpStatus.OK);
               // return new ResponseEntity(orgRepo.findById(parentId), HttpStatus.OK);
            }
        }
        return new ResponseEntity(orgRepo.findByParentId(parentId), HttpStatus.OK);
    }

    @ApiOperation(value = "查询下属机构不带分页", notes = "不分页")
    @GetMapping("/sub/both/{parentId}")
    public ResponseEntity getBothOrg(@PathVariable Long parentId,HttpServletRequest request) {
        cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
        if(accessUser != null){
            if(accessUser.getRootOrgId().longValue() != accessUser.getOrgId().longValue()){
                return new ResponseEntity(orgRepo.findById(parentId), HttpStatus.OK);
            }
        }
        return new ResponseEntity(orgRepo.findBothByParentId(parentId), HttpStatus.OK);
    }

    @ApiOperation(value = "查询下属机构分页带查询", notes = "分页")
    @GetMapping("/sub/{parentId}/{curPage}/{pageSize}")
    public ResponseEntity getAllSubOrg(@ModelAttribute Org orgCriteria,
                                       @PathVariable Long parentId,
                                       @PathVariable Integer curPage,
                                       @PathVariable Integer pageSize) {

        orgCriteria.setParentId(parentId);
        return new ResponseEntity(orgService.findAll(orgCriteria, new PageRequest(curPage - 1, pageSize)), HttpStatus.OK);
    }

//    @ApiOperation(value="按rootId查询下属机构不带分页",notes="下属机构查询不带分页")
//    @GetMapping("/allSub/{rootId}")
//    public ResponseEntity<Org> getOrgByRootId(@PathVariable Long rootId){
//        return new ResponseEntity(orgRepo.findByRootId(rootId),HttpStatus.OK);
//    }

    @ApiOperation(value = "新增机构", notes = "新增")
    @PostMapping
    public ResponseEntity addSchool(@RequestBody Org org, HttpServletRequest request) {
        org.setCreateTime(new Date());
        cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
        if (accessUser != null) {
            org.setParentId(accessUser.getRootOrgId());
            org.setRootId(accessUser.getOrgId());
        }
        try {
            return new ResponseEntity(orgService.save(org), HttpStatus.CREATED);
        } catch (Exception e) {
            e.printStackTrace();
            return new ResponseEntity(new ErrorMsg(e.getMessage()), HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @ApiOperation(value = "更新机构", notes = "更新")
    @PutMapping
    public ResponseEntity updateSchool(@RequestBody Org org, HttpServletRequest request) {
        cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
        if (accessUser != null) {
            org.setParentId(accessUser.getOrgId());
            org.setRootId(accessUser.getRootOrgId());
        }
        try {
            return new ResponseEntity(orgService.update(org.getId(), org), HttpStatus.OK);
        } catch (Exception e) {
            return new ResponseEntity(new ErrorMsg(e.getMessage()), HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @ApiOperation(value = "按ID删除机构", notes = "删除")
    @DeleteMapping("/{id}")
    public ResponseEntity deleteSchool(@PathVariable Long id) {
        orgRepo.delete(id);
        return new ResponseEntity(HttpStatus.OK);
    }

    @ApiOperation(value = "按父ID导入子机构", notes = "导入子机构")
    @PostMapping("/import")
    public ResponseEntity<?> importLearnCenter(HttpServletRequest request,
                                            @RequestParam CommonsMultipartFile file) {
        cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
        Long orgId = null;
        if(accessUser != null){
            orgId = accessUser.getRootOrgId();
        }else{
            return new ResponseEntity<>(new ErrorMsg("用户token不存在或已失效"),HttpStatus.NOT_FOUND);
        }
        try {
            List<ExcelError> excelErrors = orgService.importLearnCenter(orgId, file.getInputStream());
            return new ResponseEntity<>(excelErrors, HttpStatus.OK);
        } catch (IOException e) {
        	log.error(e.getMessage(), e);
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @ApiOperation(value="下载导入模板",notes = "下载导入模板")
    @GetMapping("/download")
    public void importFileTemplate(HttpServletResponse response){
        List<OrgDto> list= new ArrayList<OrgDto>();
        ExportService.exportEXCEL("学习中心导入模板", OrgDto.class, list, response);
    }

    @ApiOperation(value = "启用机构", notes = "启用")
    @PutMapping("/enable/{ids}")
    public ResponseEntity enableSchool(@PathVariable String ids) {
        List<Long> orgIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        for (Long orgId : orgIds) {
            orgService.enableSchool(orgId, true);
        }
        return new ResponseEntity(HttpStatus.OK);
    }

    @ApiOperation(value = "禁用机构", notes = "禁用")
    @PutMapping("/disable/{ids}")
    public ResponseEntity disableSchool(@PathVariable String ids) {
        List<Long> orgIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        for (Long orgId : orgIds) {
            orgService.enableSchool(orgId, false);
        }
        return new ResponseEntity(HttpStatus.OK);
    }

    @ApiOperation(value = "按code查询机构(学习中心)", notes = "code查询")
    @GetMapping()
    public ResponseEntity getCourseByCode(@RequestParam Long parentId, @RequestParam String code) {
    	Org org = orgRepo.findFirstByParentIdAndCode(parentId, code);
        return new ResponseEntity(org, HttpStatus.OK);
    }

    @ApiOperation(value = "按code查询机构(学习中心)", notes = "code查询")
    @GetMapping("/logo")
    public void logo(@RequestParam("domain") String domain, HttpServletResponse response) {
        Org org = orgRepo.findFirstByParentIdAndCode(0L, domain);

        String logo = org.getLogo();
        try {
            IOUtils.copy(new FileInputStream(logo), response.getOutputStream());
            response.flushBuffer();
        } catch (IOException e) {
            LOG.error("下载LOGO失败，", e);
        }

    }

    
    @ApiOperation(value = "查询机构不分页带查询", notes = "不分页带查询")
    @GetMapping("/all")
    public ResponseEntity getAllExam(@ModelAttribute Org orgCriteria,
                                     HttpServletRequest request) {
        cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
        if (accessUser != null) {
            return new ResponseEntity(orgRepo.findAllSchoolByParentId(accessUser.getRootOrgId()), HttpStatus.OK);
        }
        return new ResponseEntity(orgService.findAll(orgCriteria), HttpStatus.OK);
    }
    
    
    @ApiOperation(value = "查询所有学校", notes = "查询所有学校")
    @GetMapping("/getAllParentOrg")
    public ResponseEntity getAllParentOrg(){
    	return new ResponseEntity(orgRepo.findAllParentOrg(), HttpStatus.OK);
    }

    @ApiOperation(value = "查询所有印刷机构", notes = "查询所有印刷机构")
    @GetMapping("/allPrint")
    public ResponseEntity getAllPrintOrg() {
        return new ResponseEntity(orgRepo.findAllPrintByParentId(0L), HttpStatus.OK);
    }
    
	@ApiOperation(value = "导入logo", notes = "导入logo")
	@PostMapping("/importLogo/{id}")
	public ResponseEntity importLogo(@PathVariable Long id, HttpServletRequest request,
			@RequestParam CommonsMultipartFile file) {
		try {
			String filePath = rootPath + "/logo/" + id + ".jpg";
			FileUtils.copyInputStreamToFile(file.getInputStream(), new File(filePath));
			jdbcTemplate.update("update ecs_core_org t set  t.logo=? where t.id=?", filePath, id);
			return new ResponseEntity("", HttpStatus.OK);
		} catch (Exception e) {
			return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}
	
	@ApiOperation(value = "按机构名称模糊查询机构列表", notes = "")
	@PostMapping("getOrgListByNameLike")
	public List<Org> getOrgListByNameLike(@RequestParam String orgName) {
		User accessUser = getAccessUser();
		orgName = "%" + orgName + "%";
		List<Org> list = orgRepo.findByRootIdAndNameLike(accessUser.getRootOrgId(), orgName);
		return list;
	}
	
}
