package cn.com.qmth.examcloud.core.basic.api.provider;

import java.util.List;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.security.bean.Role;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.RolePrivilegeCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.PrivilegeInfo;
import cn.com.qmth.examcloud.core.basic.api.bean.RoleBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetPrivilegesReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetPrivilegesResp;
import cn.com.qmth.examcloud.core.basic.dao.PrivilegeGroupRepo;
import cn.com.qmth.examcloud.core.basic.dao.PrivilegeRepo;
import cn.com.qmth.examcloud.core.basic.dao.RolePrivilegeRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.RoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.Privilege;
import cn.com.qmth.examcloud.core.basic.dao.entity.PrivilegeGroup;
import cn.com.qmth.examcloud.core.basic.dao.entity.RolePrivilegeRelation;
import io.swagger.annotations.ApiOperation;

/**
 * 角色权限服务
 *
 * @author WANGWEI
 * @date 2018年6月8日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@RequestMapping("${$rmp}" + "rolePrivilege")
public class RolePrivilegeCloudServiceProvider extends ControllerSupport
		implements
			RolePrivilegeCloudService {

	private static final long serialVersionUID = -4360164791713797878L;

	@Autowired
	RoleRepo roleRepo;

	@Autowired
	RolePrivilegeRelationRepo rolePrivilegeRelationRepo;

	@Autowired
	PrivilegeRepo privilegeRepo;

	@Autowired
	PrivilegeGroupRepo privilegeGroupRepo;

	@ApiOperation(value = "查询用户权限", notes = "")
	@PostMapping("getPrivileges")
	@Override
	public GetPrivilegesResp getPrivileges(@RequestBody GetPrivilegesReq req) {

		User accessUser = getAccessUser();
		String groupCode = req.getGroupCode();
		Long rootOrgId = Long.parseLong(req.getRootOrgId());

		PrivilegeGroup privilegeGroup = privilegeGroupRepo.findByCode(groupCode);

		if (null == privilegeGroup) {
			throw new StatusException("B-002001", "groupCode is not existing");
		}

		List<Role> roleList = accessUser.getRoleList();
		List<Long> roleIdList = Lists.newArrayList();
		for (Role cur : roleList) {
			String roleCode = cur.getRoleCode();
			cn.com.qmth.examcloud.core.basic.dao.entity.Role role = roleRepo.findByCode(roleCode);
			if (null == role) {
				throw new StatusException("B-002002", "role code is wrong. roleCode=" + roleCode);
			}
			roleIdList.add(role.getId());
		}

		List<RolePrivilegeRelation> rolePrivRelationList = rolePrivilegeRelationRepo
				.findAllByRoleIdInAndRootOrgId(roleIdList, rootOrgId);

		Set<String> pIdSet = Sets.newHashSet();
		for (RolePrivilegeRelation cur : rolePrivRelationList) {
			pIdSet.add(String.valueOf(cur.getPrivilegeId()));
		}

		List<Privilege> privilegeList = privilegeRepo.findAllByGroupId(privilegeGroup.getId());

		List<PrivilegeInfo> privilegeInfoList = Lists.newArrayList();

		boolean full = req.isFull();

		for (Privilege cur : privilegeList) {
			boolean hasPrivilege = pIdSet.contains(String.valueOf(cur.getId()));
			if ((!full) && (!hasPrivilege)) {
				continue;
			}

			PrivilegeInfo privilegeInfo = new PrivilegeInfo();
			privilegeInfo.setHasPrivilege(hasPrivilege);
			privilegeInfo.setCode(cur.getCode());
			privilegeInfo.setCreationTime(cur.getCreationTime());
			privilegeInfo.setGroupId(cur.getGroupId());
			privilegeInfo.setId(cur.getId());
			privilegeInfo.setName(cur.getName());
			privilegeInfo.setParentId(cur.getParentId());
			privilegeInfo.setUpdateTime(cur.getUpdateTime());
			privilegeInfo.setDescription(cur.getDescription());
			privilegeInfo.setExt1(cur.getExt1());
			privilegeInfo.setExt2(cur.getExt2());
			privilegeInfo.setExt3(cur.getExt3());
			privilegeInfo.setExt4(cur.getExt4());
			privilegeInfo.setExt5(cur.getExt5());

			privilegeInfoList.add(privilegeInfo);

		}

		GetPrivilegesResp resp = new GetPrivilegesResp();
		resp.setPrivilegeInfoList(privilegeInfoList);
		return resp;
	}

	@ApiOperation(value = "查询所有角色", notes = "")
	@PostMapping("getAllRoles")
	@Override
	public List<RoleBean> getAllRoles() {
		List<cn.com.qmth.examcloud.core.basic.dao.entity.Role> roleList = roleRepo.findAll();
		List<RoleBean> roleBeanList = Lists.newArrayList();
		for (cn.com.qmth.examcloud.core.basic.dao.entity.Role cur : roleList) {
			RoleBean bean = new RoleBean();
			bean.setRoleId(cur.getId());
			bean.setRoleName(cur.getName());
			bean.setRoleCode(cur.getCode());

			roleBeanList.add(bean);
		}

		return roleBeanList;
	}

}
