package cn.com.qmth.examcloud.core.basic.dao;

import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.data.repository.query.QueryByExampleExecutor;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.examcloud.core.basic.dao.entity.User;

/**
 * Created by songyue on 17/1/13.
 */
public interface UserRepo extends JpaRepository<User,Long>,QueryByExampleExecutor<User>,JpaSpecificationExecutor<User>{

    User findByRootOrgIdAndLoginName(long rootOrgId,String loginName);

    User findByLoginName(String loginName);

    @Transactional
    @Modifying
    @Query("update User set password = :password where id = :id")
    void updatePasswordById(@Param("id")long id,@Param("password")String password);

    int countByLoginName(String loginName);

    List<User> findByOrgIdAndType(long orgId,String userType);

    List<User> findByRootOrgIdAndType(long rootOrgId,String userType);

	List<User> findByOrgId(Long orgId);
	
	@Query(nativeQuery = true,value = "SELECT DISTINCT * FROM ecs_core_user us, ecs_core_user_role ur WHERE us.id = ur.user_id And us.root_org_id = ? And ur.role_code = 'MARKER' And us.enable = 1")
	List<User> findMarkerByRootOrgId(Long rootOrgId);

    @Query(value = "SELECT distinct * FROM ecs_core_user us, ecs_core_user_role ur WHERE us.id = ur.user_id " +
            "And us.root_org_id = ? And ur.role_code = 'MARKER' And us.enable = 1 ORDER BY ?#{#pageable}",
    countQuery = "SELECT count(distinct us.id) FROM ecs_core_user us, ecs_core_user_role ur WHERE us.id = ur.user_id " +
            "And us.root_org_id = ? And ur.role_code = 'MARKER' And us.enable = 1 ORDER BY ?#{#pageable}",nativeQuery = true)
    Page<User> findMarkerByRootOrgId(Long rootOrgId, Pageable pageable);

    @Query(nativeQuery = true,value = "SELECT DISTINCT * FROM ecs_core_user us, ecs_core_user_role ur WHERE us.type = 'PRINT' and us.id = ur.user_id and ur.role_code = 'PROJECT_MANAGER' And us.enable = 1")
    List<User> findPrintPm();
}
