package cn.com.qmth.examcloud.core.basic.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.core.basic.dao.PermissionRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.Permission;
import cn.com.qmth.examcloud.core.basic.dao.entity.RolePermissionRelation;
import cn.com.qmth.examcloud.core.basic.dao.enums.PermissionType;

/**
 * 
 * @author  	chenken
 * @date    	2017年6月15日下午3:06:45
 * @company 	QMTH
 * @description	权限service
 */
@Service("permissionService")
public class PermissionServiceImpl implements PermissionService{
	
	@Autowired
	private PermissionRepo permissionRepo;
	
	@Autowired
	private RolePermissionRelationService rolePermissionRelationService;
	
	@Override
	public void savePermission(Permission permission) {
		permissionRepo.save(permission);
	}

	@Override
	public Map<String,List<Permission>> findPermissionsBySystem(String systemName) {
		Map<String,List<Permission>> returnMap = new HashMap<String, List<Permission>>();
		String menuType = PermissionType.MENU.name();
		returnMap.put(menuType, permissionRepo.findBySystemNameAndType(systemName,menuType));
		String pageType = PermissionType.PAGE.name();
		returnMap.put(pageType, permissionRepo.findBySystemNameAndType(systemName,pageType));
		return returnMap;
	}

	@Override
	public Permission findById(int id) {
		return permissionRepo.findOne(id);
	}

	@Override
	public List<Permission> findPermissionByIds(List<Integer> ids) {
		return permissionRepo.findByIdIn(ids);
	}

	@Override
	public List<Permission> findByPermission(Permission permission) {
		return permissionRepo.findAll(Example.of(permission));
	}

	//by weiwenhai 根据id删除权限
	@Override
	public void deletePermissionById(Integer id) {
		permissionRepo.delete(id);
	}

	//根据父级权限id查询所有子权限
	@Override
	public List<Permission> findPermissionByParentId(Integer id) {
		return permissionRepo.findByParentId(id);
	}

	@Override
	public Map<String, String> deletePermission(Integer id) {
		Map<String,String> map = new HashMap<String,String>();
		List<RolePermissionRelation> rolePermissionRelations = rolePermissionRelationService.findRolePermissionRelationByPermissonId(id);
		//首先判断该权限能否删除
		if(rolePermissionRelations.size()>0){
			map.put("message", "该权限在使用中，无法删除");
			map.put("messageType", "warning");
			return map;
		}else{
			Permission permission = findById(id);
			//然后判断该权限是否有父级
			if(permission.getParentId()!= null){
				deletePermissionById(id);
				map.put("message", "删除成功");
				map.put("messageType", "success");
				return map;
			}else{
				//最后判断父级下的权限是否删除
				List<Permission> permission_son = findPermissionByParentId(id);
				if(permission_son.size()>0){
					map.put("message", "存在子级菜单，无法删除");
					map.put("messageType", "warning");
					return map;
				}
				deletePermissionById(id);
				map.put("message", "删除成功");
				map.put("messageType", "success");
				return map;
			}
		}
	}
	
}
