package cn.com.qmth.examcloud.core.basic.starter.config;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;

import cn.com.qmth.examcloud.commons.base.util.RegExpUtil;
import cn.com.qmth.examcloud.commons.web.interceptor.FirstInterceptor;
import cn.com.qmth.examcloud.commons.web.redis.RedisClient;
import cn.com.qmth.examcloud.commons.web.security.RequestPermissionInterceptor;
import cn.com.qmth.examcloud.commons.web.security.bean.Role;

/**
 * 默认WebMvcConfigurer
 *
 * @author WANGWEI
 * @date 2018年5月22日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Configuration
public class DefaultWebMvcConfigurerAdapter extends WebMvcConfigurerAdapter {

	@Autowired
	RedisClient redisClient;

	private static Map<String, String[]> rolePathMap = new HashMap<String, String[]>();

	@Override
	public void addInterceptors(InterceptorRegistry registry) {
		registry.addInterceptor(new FirstInterceptor()).addPathPatterns("/**");
		registry.addInterceptor(getRequestPermissionInterceptor()).addPathPatterns("/**");
		super.addInterceptors(registry);
	}

	@Bean
	public RequestPermissionInterceptor getRequestPermissionInterceptor() {
		String[] exclusions = new String[]{".*login.*", ".*\\[getLoginUser\\].*",
				RegExpUtil.escape("[${app.api.root}/org]:[/download]:[GET]"),
				RegExpUtil.escape("[${app.api.root}/course]:[/download]:[GET]"),
				RegExpUtil.escape("[${app.api.root}/course]:[]:[GET]"),
				RegExpUtil.escape("[${app.api.root}/org]:[/{id}]:[GET]"),
				RegExpUtil.escape("[${app.api.root}/org]:[/all]:[GET]"),
				RegExpUtil.escape("[${app.api.root}/student]:[/{id}]:[GET]"),
				RegExpUtil.escape("[${app.api.root}/studentFaceInfo]:[/identityNumber]:[GET]"),
				RegExpUtil.escape("[${app.api.root}/user]:[/{id}]:[GET]"),
				RegExpUtil.escape("[${app.api.root}/org]:[/logo]:[GET]"),
				RegExpUtil.escape("[${$rmp}demo]:[getXxx,getYYY]:[POST]")
				//
		};

		return new RequestPermissionInterceptor(redisClient, exclusions) {

			@Override
			public boolean hasPermission(String mappingPath, List<Role> roleList) {
				String[] roles = rolePathMap.get(mappingPath);
				if (roles == null) {
					return true;
				}
				if (roleList == null) {
					return false;
				}
				List<String> roleCodes = new ArrayList<String>();
				for (Role role : roleList) {
					roleCodes.add(role.getRoleCode());
				}
				for (String role : roles) {
					if (roleCodes.contains(role)) {
						return true;
					}
				}
				return false;
			}

		};
	}
}
